<?php
 

namespace addon\wechatexpress\model;

use app\model\BaseModel;
use think\facade\Db;
use app\model\shop\Shop as ShopModel;
use app\model\order\OrderCommon;
use app\model\member\Member;
use app\model\order\OrderStock;
use app\model\goods\Goods as GoodsModel;
use app\model\store\Store as StoreModel;
use addon\wechat\model\Config as WechatConfigModel;
use Exception;

class Order extends OrderCommon
{
    
     /*****************************************************************************************订单基础状态（其他使用）********************************/
    // 订单待付款
    const ORDER_CREATE = 0;

    // 订单已支付(待发货)
    const ORDER_PAY = 1;

    // 订单备货中
    const ORDER_PENDING_DELIVERY = 2;

    // 订单已发货（配货）
    const ORDER_DELIVERY = 3;

    // 订单已收货
    const ORDER_TAKE_DELIVERY = 4;

    // 订单已结算完成
    const ORDER_COMPLETE = 10;

    // 订单已关闭
    const ORDER_CLOSE = -1;

    /***********************************************************************************订单项  配送状态**************************************************/
    // 待发货
    const DELIVERY_WAIT = 0;

    // 已发货
    const DELIVERY_DOING = 1;

    // 已收货
    const DELIVERY_FINISH = 2;
    
    /**
     * 订单类型
     *
     * @var int
     */
    public $order_type = 3;
 /**
     * 配送状态
     */
    public $delivery_status = [
        self::DELIVERY_WAIT => [
            'status' => self::DELIVERY_WAIT,
            'name' => '待发货',
            'color' => ''
        ],
        self::DELIVERY_DOING => [
            'status' => self::DELIVERY_DOING,
            'name' => '已发货',
            'color' => ''
        ],
        self::DELIVERY_FINISH => [
            'status' => self::DELIVERY_FINISH,
            'name' => '已收货',
            'color' => ''
        ]
    ];
 
   /**
     * 订单详情
     * @param $order_info
     */
    public function orderDetail($data)
    {
        $order = model('order')->getInfo([ [ 'order_id', '=', $data['order_id'] ] ]);
        $member = model('member')->getInfo([ [ 'member_id', '=', $order[ 'member_id' ] ] ], 'weapp_openid');
        $order['openid'] = $member[ 'weapp_openid' ];
        $field = '*';
        $order_common_model = new OrderCommon();
        $goods_model = new GoodsModel();
        
        $condition = array (
                [ 'order_id', '=', $data['order_id'] ],
                [ 'site_id', '=', $data['site_id'] ],
        );
        if(!empty($data['order_goods_ids'])){
            $order_goods_id_array = explode(',', $data['order_goods_ids']);
            $condition[] = ['order_goods_id', 'in', $order_goods_id_array];
        }
        
        $order_goods_list_result = $order_common_model->getOrderGoodsList($condition, $field, '', null, 'delivery_no');
        $goods_list = $order_goods_list_result[ 'data' ];
        $weight = $cargo_num = $cargo_price = 0;
        $cargo_name = '';
       
        foreach ($goods_list as &$item) {
             $goods = model('goods_sku')->getInfo([ [ 'sku_id', '=', $item['sku_id'] ], [ 'goods_id', '=', $item['goods_id'] ] ], 'weight');
             $item['weight'] = $goods['weight'];
             $weight += $goods['weight'];
             $cargo_num += $item['num'];
             $cargo_price += $item['goods_money'];
             $cargo_name = empty($cargo_name) ? $item['sku_name']:$item['sku_name'].'等';
        }
        $order['cargo_weight'] = $weight;
        $order['goods_list'] = $goods_list;
        $order['cargo_name'] = $cargo_name;
        $order['cargo_num'] = $cargo_num;
        $order['cargo_price'] = $cargo_price*100;
        $store_model = new StoreModel();
        $order['store_info'] = $store_model->getStoreInfo([ [ 'store_id', '=', $order['store_id'] ] ], '*')[ 'data' ];

        return $order;
     }
    


    public function addOrder( $param ){
        $order_id = $param[ 'order_id' ] ?? 0;
        $site_id = $param[ 'site_id' ];
        $delivery_id = $param[ 'delivery_id' ];
        $type = $param['type'] ?? 1;    //1 商家发单 2 客户发单
        $order_goods_ids = $param['order_goods_ids'] ?? '';    //1 商家发单 2 客户发单
        $weapp = new Weapp($site_id);
        $res = $weapp->getConfig($site_id);
        $data = [
            'order_id' => $order_id,
            'site_id'  => $site_id,
            'order_goods_ids' => $order_goods_ids
        ];
        $order_info = $this->orderDetail($data);
        $delivery_info = model('store_express')->getInfo([ ['delivery_id','=', $delivery_id ] ] );
        $order_from = $order_info['order_from'] == 'weapp' ? 0:2; //0 0 小程序  2 H5 或者APP
        $wechat_config_model = new WechatConfigModel();
        $wechat_config = $wechat_config_model->getWechatConfig($site_id);
        $wechat_config = $wechat_config[ "data" ][ "value" ];

        if(strpos($order_info['full_address'],'-')){
            $address = explode("-", $order_info['full_address']);//收件人
        }else{
            $address = explode(",", $order_info['full_address']);//收件人
        }
        $condition = [];
        $condition[] = ['store_id','=',$order_info['store_id']];

        
        
        /**店铺信息**/
        $shop_model = new ShopModel();
        $condition = array (
            [ "site_id", "=", $site_id ]
        );
        $shop_info_result = $shop_model->getShopInfo($condition);
        $shop_info = $shop_info_result['data'];
        $shop_info = [
            "name"=>$shop_info['name'],
            "tel"=>$shop_info['telephone'],
            "mobile"=>$shop_info['mobile'],
            "company"=>'',
            "post_code"=>'',
            "country"=>'中国',
            "province"=>$shop_info['province_name'],
            "city"=>$shop_info['city_name'],
            "area"=>$shop_info['district_name'],
            "address"=>$shop_info['address'],
        ];
        $custom_info = [
            "name"=>$order_info['name'],
            "tel"=>$order_info['telephone'],
            "mobile"=>$order_info['mobile'],
            "company"=>"",
            "post_code"=>"",
            "country"=>'中国',
            "province"=>$address[0],
            "city"=>$address[1],
            "area"=>$address[2],
            "address"=>$order_info['address'],
        ];
        if($type == 1){
            $sender = $shop_info;
            $receiver = $custom_info;
        }else{
            $sender = $custom_info;
            $receiver = $shop_info;
        }
        $insured = [
            'use_insured' => 0,
            'insured_value' => 0
        ];
        $service = [
            'service_type' => $delivery_info['service_type'],
            'service_name' => $delivery_info['service_name']
        ];
        $detail_list = $item_list=[];
        $goods_list = $order_info['goods_list'];
        $count = count($goods_list);
        foreach ($goods_list as $k => $item) {
            $item_list[$k]['goods_name'] = $item['sku_name'];
            $item_list[$k]['goods_img_url'] = $this->get_image($item['sku_image']);
            $item_list[$k]['goods_desc'] = $item['sku_name'];
            $detail_list[$k]['name'] = $item['sku_name'];
            $detail_list[$k]['count'] = $item['num'];
        }
        $shop = [
            'wxa_path' => '/pages/order/list',
            'detail_list' => $item_list
        ];
        $cargo = [
            "count"=>$count,
            "weight"=> 1,
            "space_x"=> 20,
            "space_y"=> 20,
            "space_z"=> 20,
            "detail_list"=>$detail_list
        ];
        $express_info = [
            'order_id' => $order_info['order_no'],
            'openid' => $order_info['openid'],
            'delivery_id' => $delivery_id,
            'biz_id' => $delivery_info['biz_id'],
            'custom_remark' => '',
            'add_source' => $order_from,
            'wx_appid' => $order_from == 0 ? '':$wechat_config['appid'],
            'expect_time' => '',
            'take_mode' => '',
            'sender' => $sender,
            'receiver' => $receiver,
            'insured' => $insured,
            'service' => $service,
            'shop' => $shop,
            'cargo' => $cargo,
        ];
        $res =  $weapp->addOrder($express_info);
        return $res;
    }


    
    //判断图片是否是本地，本地的加域名
    public function get_image($url){
        if($url){
            if(strstr($url, 'https://') || strstr($url, 'http://')){
                return $url;
            }else{
                $http_type = ((isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') || (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https')) ? 'https://' : 'http://';
    
                $urls = $http_type . $_SERVER['HTTP_HOST'];
                return $urls.'/'.$url;
            }
        }else{
            return '';
        }
    }


     /**
     * 订单项发货（物流）
     * @param $param
     * @param int $type 1：订单项发货，2：整体发货
     * @param array $log_data
     * @return array|int
     */
    public function orderGoodsDelivery($param, $type = 1, $log_data = [])
    {
        $param['type'] = $param['type'] ?? 'manual';
        model('order_goods')->startTrans();
        try {
            $order_id = $param['order_id'];
            $delivery_no = $param['delivery_no']; // 物流单号
            $delivery_type = $param['delivery_type'];
            if ($delivery_type == 0) {
                $express_company_id = 0;
            } else {
                $delivery_id = $param['express_company_id'] ?? '';
            }
            $express_company_name = '';
            $site_id = $param['site_id'];
            if ($type == 1) {
                if (empty($param['order_goods_ids']))
                    return $this->error('', '发货货物不可为空！');

                $order_goods_id_array = explode(',', $param['order_goods_ids']);

            } else {
                $order_goods_id_array = model('order_goods')->getColumn(
                    [
                        ['order_id', '=', $order_id],
                        ['site_id', '=', $site_id],
                        ['delivery_status', '=', self::DELIVERY_WAIT],
                        ['refund_status', '<>', 3]
                    ],
                    'order_goods_id'
                );
            }
            if (empty($order_goods_id_array))
                return $this->error('', '发货货物不可为空！');

            $order_id = 0;
            $member_id = 0;
            $goods_id_array = [];
            $order_stock_model = new OrderStock();
            $stock_sku_list = [];
            foreach ($order_goods_id_array as $k => $v) {

                $order_goods_info = model('order_goods')->getInfo([['order_goods_id', '=', $v], ['site_id', '=', $site_id]], '*');
                // 已退款的订单项不可发货
                if ($order_goods_info['refund_status'] == 3) {
                    model('order_goods')->rollback();
                    return $this->error([], 'ORDER_GOODS_IS_REFUND');
                }

                if ($order_goods_info['delivery_status'] == self::DELIVERY_DOING) {
                    model('order_goods')->rollback();
                    return $this->error([], 'ORDER_GOODS_IS_DELIVERYED');
                }
                $order_goods_info['num'] = numberFormat($order_goods_info['num']);
                $member_id = $order_goods_info['member_id'];
                $goods_id_array[] = $order_goods_info['sku_id'] . ':' . $order_goods_info['num'] . ':' . $order_goods_info['sku_name'] . ':' . $order_goods_info['sku_image'];
                $data = ['delivery_status' => self::DELIVERY_DOING, 'delivery_status_name' => $this->delivery_status[self::DELIVERY_DOING]['name']];
                if (!empty($delivery_no)) {
                    $data['delivery_no'] = $delivery_no;
                }
                $res = model('order_goods')->update($data, [
                    ['order_goods_id', '=', $v],
                    ['delivery_status', '=', self::DELIVERY_WAIT]
                ]);
                $order_id = $order_goods_info['order_id'];
                //实际发货扣除库存
                $stock_sku_list[] = $order_goods_info;

            }

            $order_info = model('order')->getInfo([['order_id', '=', $order_id]], 'store_id,site_id');
            $stock_result = $order_stock_model->decOrderStock([
                'store_id' => $order_info['store_id'],
                'site_id' => $order_info['site_id'],
                'goods_sku_list' => $stock_sku_list,
                'user_info' => $param['user_info'] ?? []
            ]);

            if ($stock_result['code'] < 0) {
                model('order_goods')->rollback();
                return $stock_result;
            }
            $res = $this->orderCommonDelivery($order_id, $log_data);
            if ($res['code'] < 0) {
                model('order_goods')->rollback();
                return $res;
            }
            $member_model = new Member();
            $member_info_result = $member_model->getMemberInfo([ [ 'member_id', '=', $member_id ] ], 'nickname');
            $member_info = $member_info_result[ 'data' ];
            if ($delivery_type > 0) {
                $count = model('express_delivery_package')->getCount([ [ 'site_id', '=', $site_id ], [ 'order_id', '=', $order_id ], [ 'delivery_type', '=', $delivery_type ] ]);
                $num = $count + 1;
                $package_name = '包裹' . $num;
                $express_company =  model('store_express')->getInfo([ [ 'delivery_id', '=', $delivery_id ] ]);
                $express_company_name = $express_company['delivery_name'];
                $express_company_id = $express_company['id'];
                $express_param = [
                    'order_id'  => $order_id,
                    'site_id'   => $order_info['site_id'],
                    'type'      => 1,
                    'order_goods_ids' => $param['order_goods_ids'],
                    'delivery_id' => $delivery_id
                ];
    
                $delivery_info = $this->addOrder($express_param);
                if ($delivery_info['code'] < 0) {
                    model('order_goods')->rollback();
                    return $delivery_info;
                }
                $delivery_no = $delivery_info[ 'data' ]['waybill_id'];
                foreach ($order_goods_id_array as $k => $v) {
                    $data = ['delivery_no' => $delivery_no];
                    $res = model('order_goods')->update($data, [
                        ['order_goods_id', '=', $v],
                        ['delivery_status', '=', self::DELIVERY_WAIT]
                    ]);
                }
            } else {
                $package_name = '无需物流';
            }
            $express_company_image = '';
            $delivery_data = array (
                'order_id' => $order_id,
                'order_goods_id_array' => implode(',', $order_goods_id_array),
                'goods_id_array' => implode(',', $goods_id_array),
                'delivery_no' => $delivery_no, 
                'site_id' => $site_id,
                'member_id' => $member_id,
                'member_name' => $member_info[ 'nickname' ] ?? '',
                'delivery_type' => $delivery_type,
                'express_company_id' => $express_company_id,
                'express_company_name' => $express_company_name,
                'package_name' => $package_name,
                'delivery_time' => time(),
                'express_company_image' => $express_company_image,
                'type' => 'wechat'
            );
            $delivery_id = model('express_delivery_package')->add($delivery_data);
            model('order_goods')->commit();
            return $this->success($delivery_id);
        } catch ( Exception $e ) {
            model('order_goods')->rollback();
            return $this->error('', $e->getMessage() . $e->getFile() . $e->getLine());
        }
    }


    public function cancelorder($param){
        $order_id = $param[ 'order_id' ] ?? 0;
        $site_id = $param[ 'site_id' ];
        $express_delivery = model('express_delivery_package')->getInfo( [ ['order_id', '=', $order_id],['site_id', '=', $site_id],['type', '=', 'wechat']]);
        $res = ['code' => 0, 'message' => '操作成功'];
        if(!empty($express_delivery) && !empty($express_delivery['delivery_no'])){ //存在配送单   且配送编号不为空
            $weapp = new Weapp($site_id);
            $res = $weapp->getConfig($site_id);
            $order_common_model = new OrderCommon();
            $order_detail = $order_common_model->getOrderDetail($order_id)[ 'data' ];
            $member = model('member')->getInfo([ [ 'member_id', '=', $order_detail[ 'member_id' ] ] ], 'weapp_openid');
            $openid = $member[ 'weapp_openid' ];
            $express_order_model = new Express();
            $company_info_result = $express_order_model->getExpressInfo([ [ 'id', '=', $express_delivery['express_company_id'] ] ]);
            $cancel_data = [
                'openid'      => $openid,
                'delivery_id' => $company_info_result[ 'data' ][ 'delivery_id' ],
                'waybill_id'  => $express_delivery['delivery_no'],
                'order_id'    => $order_detail['order_no']
            ];
            $res = $weapp->cancelorder($cancel_data);
            if($res['code'] == 0){  //清除物流单号
                $data = [
                    'delivery_no'      => ''
                ];
                model('express_delivery_package')->update($data, [ 'order_id' => $order_id ,'type' => 'wechat']);
            }
        }
        return $res;
    }

}